@@ -158,6 +158,11 @@ class JSONRPCBuilder(object):
         :rtype: string
         """
 
+        fversion = float(version)
+        if fversion >= 2.0:
+            version_field_name = 'jsonrpc'
+        else:
+            version_field_name = 'version'    
         id = int(id)
         if params:
             params_type = type(params)
@@ -165,11 +170,11 @@ class JSONRPCBuilder(object):
                 params = list(params)
             elif params_type not in [dict, list]:
                 params = [params]
-            json_str = "{\"jsonrpc\":\"%(version)s\", \"method\":\"%(method)s\", \"params\":%(params)s, \"id\":%(id)s}" \
-                        % dict(version = version, method = method, params = dumps(params), id = id)
+            json_str = "{\"%(version_field_name)s\":\"%(version)s\", \"method\":\"%(method)s\", \"params\":%(params)s, \"id\":%(id)s}" \
+                        % dict(version_field_name = version_field_name, version = version, method = method, params = dumps(params), id = id)
         else:
-            json_str = "{\"jsonrpc\":\"%(version)s\", \"method\":\"%(method)s\", \"id\":%(id)s}" \
-                        % dict(version = version, method = method, id = id)
+            json_str = "{\"%(version_field_name)s\":\"%(version)s\", \"method\":\"%(method)s\", \"id\":%(id)s}" \
+                        % dict(version_field_name = version_field_name, version = version, method = method, id = id)
         return json_str
 
     @staticmethod
@@ -184,16 +189,21 @@ class JSONRPCBuilder(object):
 
         :rtype: string
         """
-
+        
+        fversion = float(version)
+        if fversion >= 2.0:
+            version_field_name = 'jsonrpc'
+        else:
+            version_field_name = 'version'
         if error:
             if hasattr(error, 'data'):
                 json_str = '''
-                {"jsonrpc":"%(version)s", "result":null, "error":{"code":%(code)s,"message":"%(message)s", "data" : "%(data)s"}, "id":null}
-                ''' % dict(version = version, code = error.code, message = error.message or 'null', data = error.data or 'null')
+                {"%(version_field_name)s":"%(version)s", "result":null, "error":{"code":%(code)s,"message":"%(message)s", "data" : "%(data)s"}, "id":null}
+                ''' % dict(version_field_name = version_field_name, version = version, code = error.code, message = error.message or 'null', data = error.data or 'null')
             else:
                 json_str = '''
-                {"jsonrpc":"%(version)s", "result":null, "error":{"code":%(code)d,"message":"%(message)s"}, "id":null}
-                ''' % dict(version = version, code = int(error.code), message = error.message or 'null')
+                {"%(version_field_name)s":"%(version)s", "result":null, "error":{"code":%(code)d,"message":"%(message)s"}, "id":null}
+                ''' % dict(version_field_name = version_field_name, version = version, code = int(error.code), message = error.message or 'null')
         else:
             id = int(id)
             json_str = '''
@@ -280,7 +290,7 @@ class JSONRPCServerError(JSONRPCError):
     code = -32000
     message = "Server error."
 
-def jsonrpc_loads(data, version = "2.0"):
+def jsonrpc_loads(data, version = None):
     """
     将一个 jsonrpc 格式字符串转换为 python object 并返回
 
@@ -292,7 +302,15 @@ def jsonrpc_loads(data, version = "2.0"):
 
     jsonrpc_data = loads(data)
     assert "id" in jsonrpc_data, JSONRPCParseError("`id` field invalid")
-    assert "jsonrpc" in jsonrpc_data, JSONRPCParseError("`jsonrpc` field missing.")
+    if version:
+        fversion = float(version)
+        if fversion >= 2.0:
+            version_field_name = 'jsonrpc'
+        else:
+            version_field_name = 'version'
+        assert version_field_name in jsonrpc_data, JSONRPCParseError("`%s` field missing." % version_field_name)
+    else:
+        assert ('version' in jsonrpc_data) or ('jsonrpc' in jsonrpc_data), JSONRPCParseError("`version` or `jsonrpc` field missing.")
     if "method" in jsonrpc_data and isinstance(jsonrpc_data["method"], basestring):
         jsonrpc_data["method"] = jsonrpc_data["method"].strip()
         assert jsonrpc_data["method"], JSONRPCParseError("`method` field invalid.")
@@ -362,19 +380,29 @@ class JSONRPCService(DictMixin, object):
         if not req.method == "POST":
             self._logger.debug("request method '%s' not accepted." % req.method)
             resp = Response(JSONRPCBuilder.response(error = JSONRPCInvalidRequestError()),
-                            content_type = "%s; charset=utf-8" % JSON_MIME_TYPE)
+                            content_type = JSON_MIME_TYPE)
             return resp(environ, start_response)
+        post_data = req.body.strip()
         try:
-            jsonrpc_data = jsonrpc_loads(req.body.strip())
+            jsonrpc_data = jsonrpc_loads(post_data)
         except:
-            self._logger.debug("post data invalid.", exc_info = True)
+            self._logger.debug("post data invalid: %r" % post_data, exc_info = True)
             resp = Response(JSONRPCBuilder.response(error = JSONRPCParseError()),
-                            content_type = "%s; charset=utf-8" % JSON_MIME_TYPE)
+                            content_type = "%s" % JSON_MIME_TYPE)
+            return resp(environ, start_response)
+        if 'version' in jsonrpc_data:
+            version = jsonrpc_data['version']
+        elif 'jsonrpc' in jsonrpc_data:
+            version = jsonrpc_data['jsonrpc']
+        else:
+            self._logger.debug("the 'version' or 'jsonrpc' field missing." % post_data, exc_info = True)
+            resp = Response(JSONRPCBuilder.response(error = JSONRPCParseError()),
+                            content_type = JSON_MIME_TYPE)
             return resp(environ, start_response)
         if "method" not in jsonrpc_data:
             self._logger.debug("`method` field not found.")
-            resp = Response(JSONRPCBuilder.response(error = JSONRPCParseError()),
-                            content_type = "%s; charset=utf-8" % JSON_MIME_TYPE)
+            resp = Response(JSONRPCBuilder.response(error = JSONRPCParseError(), version = version),
+                            content_type = JSON_MIME_TYPE)
             return resp(environ, start_response)
         args = []
         kargs = {}
@@ -394,8 +422,8 @@ class JSONRPCService(DictMixin, object):
         provider = self.method_resolve(jsonrpc_data["method"])
         if provider is None:
             self._logger.debug("method '%s' not found." % jsonrpc_data["method"])
-            resp = Response(JSONRPCBuilder.response(error = JSONRPCMethodNotFoundError()),
-                            content_type = "%s" % JSON_MIME_TYPE)
+            resp = Response(JSONRPCBuilder.response(error = JSONRPCMethodNotFoundError(), version = version),
+                            content_type = JSON_MIME_TYPE)
             return resp(environ, start_response)
         
         environ["khan.web.service"] = self
@@ -404,8 +432,8 @@ class JSONRPCService(DictMixin, object):
                            % (self.get_provider_name(provider), args, kargs))
         try:
             result = provider(*args, **kargs)
-            resp = Response(JSONRPCBuilder.response(result = result),
-                            content_type = "%s" % JSON_MIME_TYPE)
+            resp = Response(JSONRPCBuilder.response(result = result, id = jsonrpc_data["id"], version = version),
+                            content_type = JSON_MIME_TYPE)
             return resp(environ, start_response)
         except HTTPException, httpe:
             self._logger.debug("`%r` raised HTTPException: %s (code: %s)",
@@ -417,24 +445,25 @@ class JSONRPCService(DictMixin, object):
             return httpe(environ, start_response)
         except TypeError, e:
             self._logger.error("calling `%r` with invalid arguments." % self.get_provider_name(provider), exc_info = True)
-            resp = Response(JSONRPCBuilder.response(error = JSONRPCInvalidParamsError()),
-                            content_type = "%s" % JSON_MIME_TYPE)
+            resp = Response(JSONRPCBuilder.response(error = JSONRPCInvalidParamsError(), 
+                                                    id = jsonrpc_data["id"], version = version),
+                            content_type = JSON_MIME_TYPE)
             return resp(environ, start_response)
         except JSONRPCError, e:
             self._logger.error("jsonrpc error.", exc_info = True)
-            resp = Response(JSONRPCBuilder.response(error = e),
-                            content_type = "%s" % JSON_MIME_TYPE)
+            resp = Response(JSONRPCBuilder.response(error = e, id = jsonrpc_data["id"], version = version),
+                            content_type = JSON_MIME_TYPE)
             return resp(environ, start_response)
         except:
             self._logger.error("`%r` raised unknown exception." % self.get_provider_name(provider), exc_info = True)
-            resp = Response(JSONRPCBuilder.response(error = JSONRPCInternalError()),
-                            content_type = "%s" % JSON_MIME_TYPE)
+            resp = Response(JSONRPCBuilder.response(error = JSONRPCInternalError(), 
+                                                    id = jsonrpc_data["id"], version = version),
+                            content_type = JSON_MIME_TYPE)
             return resp(environ, start_response)
 
     def __call__(self, environ, start_response):
         return self._wapper_app(environ, start_response)
 
-
 def make_jsonrpc_app(global_conf, **providers):
     """
     paste EGG Entry Point::
@@ -471,7 +500,7 @@ class JSONRPCClient(object):
             "Accept": "text/plain"
     }
 
-    def __init__(self, url, timeout = None, service_name = None, http_headers = None, version = "2.0", http_client = None):
+    def __init__(self, url, timeout = None, service_name = None, http_headers = None, version = "2.0", http_client = None, id = 0):
         """
         :param url: jsonrpc 服务 url, 支持 https.
         :param timeout: socket 级的 timeout(秒) 设置.
@@ -483,6 +512,7 @@ class JSONRPCClient(object):
         self._service_name = service_name
         self._http_headers = http_headers or {}
         self._version = version
+        self._id = id
 
     def add_credentials(self, username, password = None):
         """ 如果服务器要求身份验证，可使用本函数添加用户名和密码 """
@@ -492,7 +522,7 @@ class JSONRPCClient(object):
     def __getattr__(self, name):
         if (self._service_name != None):
             name = "%s.%s" % (self._service_name, name)
-        return JSONRPCClient(self._url, self._timeout, name, self._http_headers, version = self._version, http_client = self._http_client)
+        return JSONRPCClient(self._url, self._timeout, name, self._http_headers, version = self._version, http_client = self._http_client, id = self._id)
 
     def __call__(self, *args, **kargs):
         """
@@ -508,7 +538,8 @@ class JSONRPCClient(object):
             params = kargs
         else:
             params = None
-        post_data = JSONRPCBuilder.request(self._service_name, params, version = self._version)
+        self._id += 1
+        post_data = JSONRPCBuilder.request(self._service_name, params, version = self._version, id = self._id)
         resp, content = self._http_client.request(self._url, "POST", post_data, headers = http_headers)
         if resp.status != 200:
             raise JSONRPCError(resp.reason)
@@ -517,6 +548,8 @@ class JSONRPCClient(object):
             jsonrpc_data = jsonrpc_loads(content, self._version)
         except Exception, e:
             raise JSONRPCParseError("%s" % e)
+        if jsonrpc_data['id'] == self._id:
+            raise JSONRPCServerError('JSONRPC request id is `%s`, but response id is `%s`' % (self._id, jsonrpc_data['id']))
         if (jsonrpc_data["error"] != None):
             if jsonrpc_data["error"]["code"] in jsonrpc_error_map:
                 err_cls = jsonrpc_error_map[jsonrpc_data["error"]["code"]](jsonrpc_data["error"]["message"],
